/* source-sans-pro-300 - latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin */
@font-face {
  font-family: 'Source Sans Pro';
  font-style: normal;
  font-weight: 300;
  src: url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.eot'); /* IE9 Compat Modes */
  src: local('Source Sans Pro Light'), local('SourceSansPro-Light'),
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.woff2') format('woff2'), /* Super Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.woff') format('woff'), /* Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-300.svg#SourceSansPro') format('svg'); /* Legacy iOS */
}
/* source-sans-pro-regular - latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin */
@font-face {
  font-family: 'Source Sans Pro';
  font-style: normal;
  font-weight: 400;
  src: url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.eot'); /* IE9 Compat Modes */
  src: local('Source Sans Pro Regular'), local('SourceSansPro-Regular'),
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.woff') format('woff'), /* Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-regular.svg#SourceSansPro') format('svg'); /* Legacy iOS */
}
/* source-sans-pro-600 - latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin */
@font-face {
  font-family: 'Source Sans Pro';
  font-style: normal;
  font-weight: 600;
  src: url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.eot'); /* IE9 Compat Modes */
  src: local('Source Sans Pro SemiBold'), local('SourceSansPro-SemiBold'),
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.woff2') format('woff2'), /* Super Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.woff') format('woff'), /* Modern Browsers */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../fonts/source-sans-pro/source-sans-pro-v12-latin-ext_cyrillic-ext_cyrillic_greek_greek-ext_latin-600.svg#SourceSansPro') format('svg'); /* Legacy iOS */
}

/* dosis-500 - latin */
@font-face {
  font-family: 'Dosis';
  font-style: normal;
  font-weight: 500;
  src: url('../fonts/dosis/dosis-v8-latin-500.eot'); /* IE9 Compat Modes */
  src: local('Dosis Medium'), local('Dosis-Medium'),
       url('../fonts/dosis/dosis-v8-latin-500.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../fonts/dosis/dosis-v8-latin-500.woff2') format('woff2'), /* Super Modern Browsers */
       url('../fonts/dosis/dosis-v8-latin-500.woff') format('woff'), /* Modern Browsers */
       url('../fonts/dosis/dosis-v8-latin-500.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../fonts/dosis/dosis-v8-latin-500.svg#Dosis') format('svg'); /* Legacy iOS */
}

/* source-code-pro-regular - latin */
@font-face {
  font-family: 'Source Code Pro';
  font-style: normal;
  font-weight: 400;
  src: url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.eot'); /* IE9 Compat Modes */
  src: local('Source Code Pro'), local('SourceCodePro-Regular'),
       url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.woff') format('woff'), /* Modern Browsers */
       url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../fonts/source-code-pro/source-code-pro-v10-latin-regular.svg#SourceCodePro') format('svg'); /* Legacy iOS */
}
