FEATURES = "FEATURES"
SCHEDULER = "SCHEDULER"
STATS = "STATS"
TTL = "TTL"
SIDECARS = "SIDECARS"
INIT = "INIT"
CLEANING_INTERVALS = "CLEANING_INTERVALS"
COMPONENT = "COMPONENT"
TIMEOUT = "TIMEOUT"
EMAIL = "EMAIL"
USER = "USER"
